@@ -293,6 +293,12 @@ void bind_alert()
         .def_readonly("resume_data", &save_resume_data_alert::resume_data)
         ;
 
+    class_<file_completed_alert, bases<torrent_alert>, noncopyable>(
+        "file_completed_alert", no_init
+    )
+        .def_readonly("index", &file_completed_alert::index)
+        ;
+
     class_<file_renamed_alert, bases<torrent_alert>, noncopyable>(
         "file_renamed_alert", no_init
     )
