@@ -2,12 +2,12 @@
 // subject to the Boost Software License, Version 1.0. (See accompanying
 // file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
 
+#include <boost/python.hpp>
 #include <libtorrent/session.hpp>
 #include <libtorrent/torrent.hpp>
 #include <libtorrent/storage.hpp>
 #include <libtorrent/ip_filter.hpp>
 #include <libtorrent/disk_io_thread.hpp>
-#include <boost/python.hpp>
 #include "gil.hpp"
 
 using namespace boost::python;
@@ -107,6 +107,10 @@ namespace
             p.auto_managed = params["auto_managed"];
         if (params.has_key("duplicate_is_error"))
             p.duplicate_is_error = params["duplicate_is_error"];
+        if (params.has_key("seed_mode"))
+            p.seed_mode = params["seed_mode"];
+        if (params.has_key("override_resume_data"))
+            p.override_resume_data = params["override_resume_data"];
 
         return s.add_torrent(p);
     }
@@ -155,35 +159,92 @@ namespace
         return s.load_country_db(file.c_str());
     }
 #endif
+
+    entry save_state(session const& s, boost::uint32_t flags)
+    {
+        entry e;
+        s.save_state(e, flags);
+        return e;
+    }
+
 } // namespace unnamed
 
+
 void bind_session()
 {
+#ifndef TORRENT_DISABLE_DHT
+    void (session::*start_dht0)() = &session::start_dht;
+#ifndef TORRENT_NO_DEPRECATE
+    void (session::*start_dht1)(entry const&) = &session::start_dht;
+#endif
+#endif
+
+    void (session::*load_state0)(lazy_entry const&) = &session::load_state;
+#ifndef TORRENT_NO_DEPRECATE
+    void (session::*load_state1)(entry const&) = &session::load_state;
+#endif
+
     class_<session_status>("session_status")
         .def_readonly("has_incoming_connections", &session_status::has_incoming_connections)
+
         .def_readonly("upload_rate", &session_status::upload_rate)
         .def_readonly("download_rate", &session_status::download_rate)
-        .def_readonly("payload_upload_rate", &session_status::payload_upload_rate)
-        .def_readonly("payload_download_rate", &session_status::payload_download_rate)
         .def_readonly("total_download", &session_status::total_download)
         .def_readonly("total_upload", &session_status::total_upload)
+
+        .def_readonly("payload_upload_rate", &session_status::payload_upload_rate)
+        .def_readonly("payload_download_rate", &session_status::payload_download_rate)
         .def_readonly("total_payload_download", &session_status::total_payload_download)
         .def_readonly("total_payload_upload", &session_status::total_payload_upload)
+
+        .def_readonly("ip_overhead_upload_rate", &session_status::ip_overhead_upload_rate)
+        .def_readonly("ip_overhead_download_rate", &session_status::ip_overhead_download_rate)
+        .def_readonly("total_ip_overhead_download", &session_status::total_ip_overhead_download)
+        .def_readonly("total_ip_overhead_upload", &session_status::total_ip_overhead_upload)
+
+        .def_readonly("dht_upload_rate", &session_status::dht_upload_rate)
+        .def_readonly("dht_download_rate", &session_status::dht_download_rate)
+        .def_readonly("total_dht_download", &session_status::total_dht_download)
+        .def_readonly("total_dht_upload", &session_status::total_dht_upload)
+
+        .def_readonly("tracker_upload_rate", &session_status::tracker_upload_rate)
+        .def_readonly("tracker_download_rate", &session_status::tracker_download_rate)
+        .def_readonly("total_tracker_download", &session_status::total_tracker_download)
+        .def_readonly("total_tracker_upload", &session_status::total_tracker_upload)
+
         .def_readonly("total_redundant_bytes", &session_status::total_redundant_bytes)
         .def_readonly("total_failed_bytes", &session_status::total_failed_bytes)
+
         .def_readonly("num_peers", &session_status::num_peers)
         .def_readonly("num_unchoked", &session_status::num_unchoked)
         .def_readonly("allowed_upload_slots", &session_status::allowed_upload_slots)
+
         .def_readonly("up_bandwidth_queue", &session_status::up_bandwidth_queue)
         .def_readonly("down_bandwidth_queue", &session_status::down_bandwidth_queue)
+
+        .def_readonly("up_bandwidth_bytes_queue", &session_status::up_bandwidth_bytes_queue)
+        .def_readonly("down_bandwidth_bytes_queue", &session_status::down_bandwidth_bytes_queue)
+
+        .def_readonly("optimistic_unchoke_counter", &session_status::optimistic_unchoke_counter)
+        .def_readonly("unchoke_counter", &session_status::unchoke_counter)
+
 #ifndef TORRENT_DISABLE_DHT
         .def_readonly("dht_nodes", &session_status::dht_nodes)
         .def_readonly("dht_node_cache", &session_status::dht_node_cache)
         .def_readonly("dht_torrents", &session_status::dht_torrents)
         .def_readonly("dht_global_nodes", &session_status::dht_global_nodes)
+        .def_readonly("active_requests", &session_status::active_requests)
 #endif
         ;
 
+    class_<dht_lookup>("dht_lookup")
+        .def_readonly("type", &dht_lookup::type)
+        .def_readonly("outstanding_requests", &dht_lookup::outstanding_requests)
+        .def_readonly("timeouts", &dht_lookup::timeouts)
+        .def_readonly("response", &dht_lookup::responses)
+        .def_readonly("branch_factor", &dht_lookup::branch_factor)
+    ;
+
     enum_<storage_mode_t>("storage_mode_t")
         .value("storage_mode_allocate", storage_mode_allocate)
         .value("storage_mode_sparse", storage_mode_sparse)
@@ -200,7 +261,7 @@ void bind_session()
         .value("start_default_features", session::start_default_features)
     ;
 
-     class_<cache_status>("cache_status")
+    class_<cache_status>("cache_status")
         .def_readonly("blocks_written", &cache_status::blocks_written)
         .def_readonly("writes", &cache_status::writes)
         .def_readonly("blocks_read", &cache_status::blocks_read)
@@ -208,25 +269,36 @@ void bind_session()
         .def_readonly("reads", &cache_status::reads)
         .def_readonly("cache_size", &cache_status::cache_size)
         .def_readonly("read_cache_size", &cache_status::read_cache_size)
+        .def_readonly("total_used_buffers", &cache_status::total_used_buffers)
     ;
 
     class_<session, boost::noncopyable>("session", no_init)
         .def(
             init<fingerprint, int>((
                 arg("fingerprint")=fingerprint("LT",0,1,0,0)
-                , arg("flags")=session::start_default_features | session::add_default_plugins)
-                )
+                , arg("flags")=session::start_default_features | session::add_default_plugins))
+        )
+        .def(
+            "listen_on", &listen_on
+          , (arg("min"), "max", arg("interface") = (char const*)0)
         )
-        .def("listen_on", &listen_on, (arg("min"), "max", arg("interface") = (char const*)0))
         .def("outgoing_ports", &outgoing_ports)
         .def("is_listening", allow_threads(&session::is_listening))
         .def("listen_port", allow_threads(&session::listen_port))
         .def("status", allow_threads(&session::status))
 #ifndef TORRENT_DISABLE_DHT
-        .def("add_dht_router", &add_dht_router, (arg("router"), "port"))
-        .def("start_dht", allow_threads(&session::start_dht))
+        .def(
+            "add_dht_router", &add_dht_router
+          , (arg("router"), "port")
+        )
+        .def("start_dht", allow_threads(start_dht0))
+#ifndef TORRENT_NO_DEPRECATE
+        .def("start_dht", allow_threads(start_dht1))
+#endif
         .def("stop_dht", allow_threads(&session::stop_dht))
+#ifndef TORRENT_NO_DEPRECATE
         .def("dht_state", allow_threads(&session::dht_state))
+#endif
         .def("set_dht_proxy", allow_threads(&session::set_dht_proxy))
         .def("dht_proxy", allow_threads(&session::dht_proxy), return_value_policy<copy_const_reference>())
 #endif
@@ -235,16 +307,26 @@ void bind_session()
         .def(
             "add_torrent", &add_torrent_depr
           , (
-                arg("resume_data") = entry(), arg("storage_mode") = storage_mode_sparse,
+                arg("resume_data") = entry(),
+                arg("storage_mode") = storage_mode_sparse,
                 arg("paused") = false
             )
         )
 #endif
-        .def("remove_torrent", allow_threads(&session::remove_torrent), arg("option") = session::none)
+        .def("remove_torrent", allow_threads(&session::remove_torrent), arg("option") = session::none
+)
+        .def("set_local_download_rate_limit", allow_threads(&session::set_local_download_rate_limit))
+        .def("local_download_rate_limit", allow_threads(&session::local_download_rate_limit))
+
+        .def("set_local_upload_rate_limit", allow_threads(&session::set_local_upload_rate_limit))
+        .def("local_upload_rate_limit", allow_threads(&session::local_upload_rate_limit))
+
         .def("set_download_rate_limit", allow_threads(&session::set_download_rate_limit))
         .def("download_rate_limit", allow_threads(&session::download_rate_limit))
+
         .def("set_upload_rate_limit", allow_threads(&session::set_upload_rate_limit))
         .def("upload_rate_limit", allow_threads(&session::upload_rate_limit))
+
         .def("set_max_uploads", allow_threads(&session::set_max_uploads))
         .def("set_max_connections", allow_threads(&session::set_max_connections))
         .def("set_max_half_open_connections", allow_threads(&session::set_max_half_open_connections))
@@ -259,9 +341,10 @@ void bind_session()
         .def("load_asnum_db", &load_asnum_db)
         .def("load_country_db", &load_country_db)
 #endif
-        .def("load_state", allow_threads(&session::load_state))
-        .def("state", allow_threads(&session::state))
+        .def("load_state", load_state0)
+        .def("save_state", &save_state, (arg("entry"), arg("flags") = 0xffffffff))
 #ifndef TORRENT_NO_DEPRECATE
+        .def("load_state", load_state1)
         .def("set_severity_level", allow_threads(&session::set_severity_level))
 #endif
         .def("set_alert_mask", allow_threads(&session::set_alert_mask))
@@ -291,5 +374,18 @@ void bind_session()
         .def("get_cache_status", allow_threads(&session::get_cache_status))
         ;
 
+    enum_<session::save_state_flags_t>("save_state_flags_t")
+        .value("save_settings", session::save_settings)
+        .value("save_dht_settings", session::save_dht_settings)
+        .value("save_dht_proxy", session::save_dht_proxy)
+        .value("save_dht_state", session::save_dht_state)
+        .value("save_i2p_proxy", session::save_i2p_proxy)
+        .value("save_encryption_settings", session:: save_encryption_settings)
+        .value("save_peer_proxy", session::save_peer_proxy)
+        .value("save_web_proxy", session::save_web_proxy)
+        .value("save_tracker_proxy", session::save_tracker_proxy)
+        .value("save_as_map", session::save_as_map)
+    ;
+
     register_ptr_to_python<std::auto_ptr<alert> >();
 }
