@@ -322,7 +322,7 @@ int peer_index(libtorrent::tcp::endpoint addr, std::vector<libtorrent::peer_info
 {
 	using namespace libtorrent;
 	std::vector<peer_info>::const_iterator i = std::find_if(peers.begin()
-		, peers.end(), bind(&peer_info::ip, _1) == addr);
+		, peers.end(), boost::bind(&peer_info::ip, _1) == addr);
 	if (i == peers.end()) return -1;
 
 	return i - peers.begin();
@@ -562,6 +562,11 @@ void add_torrent(libtorrent::session& ses
 	p.duplicate_is_error = false;
 	p.auto_managed = true;
 	torrent_handle h = ses.add_torrent(p, ec);
+	if (ec)
+	{
+		fprintf(stderr, "failed to add torrent: %s\n", ec.message().c_str());
+		return;
+	}
 
 	handles.insert(std::pair<const std::string, torrent_handle>(
 		monitored_dir?std::string(torrent):std::string(), h));
@@ -710,7 +715,7 @@ void handle_alert(libtorrent::session& ses, libtorrent::alert* a
 			bencode(std::back_inserter(out), *p->resume_data);
 			save_file(h.save_path() / (h.name() + ".resume"), out);
 			if (std::find_if(handles.begin(), handles.end()
-				, bind(&handles_t::value_type::second, _1) == h) == handles.end())
+				, boost::bind(&handles_t::value_type::second, _1) == h) == handles.end())
 				ses.remove_torrent(h);
 		}
 	}
@@ -718,7 +723,7 @@ void handle_alert(libtorrent::session& ses, libtorrent::alert* a
 	{
 		torrent_handle h = p->handle;
 		if (std::find_if(handles.begin(), handles.end()
-			, bind(&handles_t::value_type::second, _1) == h) == handles.end())
+			, boost::bind(&handles_t::value_type::second, _1) == h) == handles.end())
 			ses.remove_torrent(h);
 	}
 }
@@ -767,6 +772,10 @@ int main(int argc, char* argv[])
 			"  -A <num pieces>       allowed pieces set size\n"
 			"  -R <num blocks>       number of blocks per read cache line\n"
 			"  -O                    Disallow disk job reordering\n"
+			"  -P <host:port>        Use the specified SOCKS5 proxy\n"
+			"  -L <user:passwd>      Use the specified username and password for the\n"
+			"                        proxy specified by -P\n"
+			"  -W <num peers>        Set the max number of peers to keep in the peer list\n"
 			"  "
 			"\n\n"
 			"TORRENT is a path to a .torrent file\n"
@@ -777,7 +786,6 @@ int main(int argc, char* argv[])
 
 	using namespace libtorrent;
 	session_settings settings;
-	proxy_settings ps;
 
 	settings.user_agent = "client_test/" LIBTORRENT_VERSION;
 	settings.auto_upload_slots_rate_based = true;
@@ -785,6 +793,8 @@ int main(int argc, char* argv[])
 	settings.optimize_hashing_for_speed = false;
 	settings.disk_cache_algorithm = session_settings::largest_contiguous;
 
+	proxy_settings ps;
+
 	int refresh_delay = 1;
 
 	std::deque<std::string> events;
@@ -832,37 +842,15 @@ int main(int argc, char* argv[])
 
 	// load the torrents given on the commandline
 
+	std::vector<add_torrent_params> magnet_links;
+	std::vector<std::string> torrents;
+
 	for (int i = 1; i < argc; ++i)
 	{
 		if (argv[i][0] != '-')
 		{
 			// interpret this as a torrent
 
-			// first see if this is a torrentless download
-			if (std::strstr("magnet:", argv[i]) == argv[i])
-			{
-				add_torrent_params p;
-				p.save_path = save_path;
-				p.storage_mode = (storage_mode_t)allocation_mode;
-				printf("adding MANGET link: %s\n", argv[i]);
-				error_code ec;
-				torrent_handle h = add_magnet_uri(ses, argv[i], p, ec);
-				if (ec)
-				{
-					fprintf(stderr, "%s\n", ec.message().c_str());
-					continue;
-				}
-
-				handles.insert(std::pair<const std::string, torrent_handle>(std::string(), h));
-
-				h.set_max_connections(max_connections_per_torrent);
-				h.set_max_uploads(-1);
-				h.set_ratio(preferred_ratio);
-				h.set_upload_limit(torrent_upload_limit);
-				h.set_download_limit(torrent_download_limit);
-				continue;
-			}
-
 			// match it against the <hash>@<tracker> format
 			if (strlen(argv[i]) > 45
 				&& is_hex(argv[i], 40)
@@ -879,28 +867,11 @@ int main(int argc, char* argv[])
 				p.paused = true;
 				p.duplicate_is_error = false;
 				p.auto_managed = true;
-				error_code ec;
-				torrent_handle h = ses.add_torrent(p, ec);
-				if (ec)
-				{
-					fprintf(stderr, "%s\n", ec.message().c_str());
-					continue;
-				}
-
-				handles.insert(std::pair<const std::string, torrent_handle>(std::string(), h));
-
-				h.set_max_connections(max_connections_per_torrent);
-				h.set_max_uploads(-1);
-				h.set_ratio(preferred_ratio);
-				h.set_upload_limit(torrent_upload_limit);
-				h.set_download_limit(torrent_download_limit);
+				magnet_links.push_back(p);
 				continue;
 			}
 
-			// if it's a torrent file, open it as usual
-			add_torrent(ses, handles, argv[i], preferred_ratio
-				, allocation_mode, save_path, false
-				, torrent_upload_limit, torrent_download_limit);
+			torrents.push_back(argv[i]);
 			continue;
 		}
 
@@ -935,32 +906,22 @@ int main(int argc, char* argv[])
 			case 'F': refresh_delay = atoi(arg); break;
 			case 'x':
 				{
-					/*
-					std::ifstream in(arg);
-					ip_filter filter;
-					while (in.good())
+					FILE* filter = fopen(arg, "r");
+					if (filter)
 					{
-						char line[300];
-						in.getline(line, 300);
-						int len = in.gcount();
-						if (len <= 0) continue;
-						if (line[0] == '#') continue;
-						int a, b, c, d;
-						char dummy;
-						std::stringstream ln(line);
-						ln >> a >> dummy >> b >> dummy >> c >> dummy >> d >> dummy;
-						address_v4 start((a << 24) + (b << 16) + (c << 8) + d);
-						ln >> a >> dummy >> b >> dummy >> c >> dummy >> d;
-						address_v4 last((a << 24) + (b << 16) + (c << 8) + d);
-						int flags;
-						ln >> flags;
-						if (flags <= 127) flags = ip_filter::blocked;
-						else flags = 0;
-						if (ln.fail()) break;
-						filter.add_rule(start, last, flags);
+						ip_filter fil;
+						unsigned int a,b,c,d,e,f,g,h, flags;
+						while (fscanf(filter, "%u.%u.%u.%u - %u.%u.%u.%u %u\n", &a, &b, &c, &d, &e, &f, &g, &h, &flags) == 9)
+						{
+							address_v4 start((a << 24) + (b << 16) + (c << 8) + d);
+							address_v4 last((e << 24) + (f << 16) + (g << 8) + h);
+							if (flags <= 127) flags = ip_filter::blocked;
+							else flags = 0;
+							fil.add_rule(start, last, flags);
+						}
+						ses.set_ip_filter(fil);
+						fclose(filter);
 					}
-					ses.set_ip_filter(filter);
-					*/
 				}
 				break;
 			case 'c': ses.set_max_connections(atoi(arg)); break;
@@ -973,15 +934,110 @@ int main(int argc, char* argv[])
 			case 'A': settings.allowed_fast_set_size = atoi(arg); break;
 			case 'R': settings.read_cache_line_size = atoi(arg); break;
 			case 'O': settings.allow_reordered_disk_operations = false; --i; break;
+			case 'W': settings.max_peerlist_size = atoi(arg); break;
+			case 'P':
+				{
+					char* port = (char*) strchr(arg, ':');
+					if (port == 0)
+					{
+						fprintf(stderr, "invalid proxy hostname, no port found\n");
+						break;
+					}
+					*port++ = 0;
+					ps.hostname = arg;
+					ps.port = atoi(port);
+					if (ps.port == 0) {
+						fprintf(stderr, "invalid proxy port\n");
+						break;
+					}
+					if (ps.type == proxy_settings::none)
+						ps.type = proxy_settings::socks5;
+				}
+				break;
+			case 'L':
+				{
+					char* pw = (char*) strchr(arg, ':');
+					if (pw == 0)
+					{
+						fprintf(stderr, "invalid proxy username and password specified\n");
+						break;
+					}
+					*pw++ = 0;
+					ps.username = arg;
+					ps.password = pw;
+					ps.type = proxy_settings::socks5_pw;
+				}
+				break;
 		}
 		++i; // skip the argument
 	}
 
+	ses.set_peer_proxy(ps);
+	ses.set_web_seed_proxy(ps);
+	ses.set_tracker_proxy(ps);
+#ifndef TORRENT_DISABLE_DHT
+	ses.set_dht_proxy(ps);
+#endif
+
 	ses.listen_on(std::make_pair(listen_port, listen_port + 10)
 		, bind_to_interface.c_str());
 
 	ses.set_settings(settings);
 
+	for (std::vector<add_torrent_params>::iterator i = magnet_links.begin()
+		, end(magnet_links.end()); i != end; ++i)
+	{
+		error_code ec;
+		torrent_handle h = ses.add_torrent(*i, ec);
+		if (ec)
+		{
+			fprintf(stderr, "failed to add torrent: %s\n", ec.message().c_str());
+			continue;
+		}
+
+		handles.insert(std::pair<const std::string, torrent_handle>(std::string(), h));
+
+		h.set_max_connections(max_connections_per_torrent);
+		h.set_max_uploads(-1);
+		h.set_ratio(preferred_ratio);
+		h.set_upload_limit(torrent_upload_limit);
+		h.set_download_limit(torrent_download_limit);
+	}
+
+	for (std::vector<std::string>::iterator i = torrents.begin()
+		, end(torrents.end()); i != end; ++i)
+	{
+		// first see if this is a torrentless download
+		if (std::strstr(i->c_str(), "magnet:") == i->c_str())
+		{
+			add_torrent_params p;
+			p.save_path = save_path;
+			p.storage_mode = (storage_mode_t)allocation_mode;
+			printf("adding MANGET link: %s\n", i->c_str());
+			error_code ec;
+			torrent_handle h = add_magnet_uri(ses, i->c_str(), p, ec);
+			if (ec)
+			{
+				fprintf(stderr, "%s\n", ec.message().c_str());
+				continue;
+			}
+
+			handles.insert(std::pair<const std::string, torrent_handle>(std::string(), h));
+
+			h.set_max_connections(max_connections_per_torrent);
+			h.set_max_uploads(-1);
+			h.set_ratio(preferred_ratio);
+			h.set_upload_limit(torrent_upload_limit);
+			h.set_download_limit(torrent_download_limit);
+			continue;
+		}
+
+		// if it's a torrent file, open it as usual
+		add_torrent(ses, handles, i->c_str(), preferred_ratio
+			, allocation_mode, save_path, false
+			, torrent_upload_limit, torrent_download_limit);
+	}
+
 	// main loop
 	std::vector<peer_info> peers;
 	std::vector<partial_piece_info> queue;
@@ -1143,6 +1199,9 @@ int main(int argc, char* argv[])
 			winsize size;
 			ioctl(STDOUT_FILENO, TIOCGWINSZ, (char*)&size);
 			terminal_width = size.ws_col;
+
+			if (terminal_width < 64)
+				terminal_width = 64;
 		}
 #endif
 
@@ -1400,10 +1459,10 @@ int main(int argc, char* argv[])
 				for (std::vector<announce_entry>::iterator i = tr.begin()
 					, end(tr.end()); i != end; ++i)
 				{
-					snprintf(str, sizeof(str), "%2d %-55s fails: %-3d %s %s\n"
-						, i->tier, i->url.c_str(), i->fails, i->verified?"OK ":"-  "
+					snprintf(str, sizeof(str), "%2d %-55s fails: %-3d (%-3d) %s %s\n"
+						, i->tier, i->url.c_str(), i->fails, i->fail_limit, i->verified?"OK ":"-  "
 						, i->updating?"updating"
-							:!i->verified?""
+							:!i->will_announce(now)?""
 							:to_string(total_seconds(i->next_announce - now), 8).c_str());
 					out += str;
 				}
@@ -1413,8 +1472,8 @@ int main(int argc, char* argv[])
 			{
 
 				h.get_download_queue(queue);
-				std::sort(queue.begin(), queue.end(), bind(&partial_piece_info::piece_index, _1)
-					< bind(&partial_piece_info::piece_index, _2));
+				std::sort(queue.begin(), queue.end(), boost::bind(&partial_piece_info::piece_index, _1)
+					< boost::bind(&partial_piece_info::piece_index, _2));
 
 				std::vector<cached_piece_info> pieces;
 				ses.get_cache_info(h.info_hash(), pieces);
@@ -1424,7 +1483,7 @@ int main(int argc, char* argv[])
 				{
 					cached_piece_info* cp = 0;
 					std::vector<cached_piece_info>::iterator cpi = std::find_if(pieces.begin(), pieces.end()
-						, bind(&cached_piece_info::piece, _1) == i->piece_index);
+						, boost::bind(&cached_piece_info::piece, _1) == i->piece_index);
 					if (cpi != pieces.end()) cp = &*cpi;
 
 					snprintf(str, sizeof(str), "%5d: [", i->piece_index);
