@@ -76,6 +76,12 @@ void print_usage()
 		"            than bytes will be piece-aligned\n"
 		"-s bytes    specifies a piece size for the torrent\n"
 		"            This has to be a multiple of 16 kiB\n"
+		"-l          Don't follow symlinks, instead encode them as\n"
+		"            links in the torrent file\n"
+		"-o file     specifies the output filename of the torrent file\n"
+		"            If this is not specified, the torrent file is\n"
+		"            printed to the standard out, except on windows\n"
+		"            where the filename defaults to a.torrent\n"
 		, stderr);
 }
 
@@ -102,6 +108,13 @@ int main(int argc, char* argv[])
 		int piece_size = 0;
 		int flags = 0;
 
+		std::string outfile;
+#ifdef TORRENT_WINDOWS
+		// don't ever write binary data to the console on windows
+		// it will just be interpreted as text and corrupted
+		outfile = "a.torrent";
+#endif
+
 		for (int i = 2; i < argc; ++i)
 		{
 			if (argv[i][0] != '-')
@@ -132,6 +145,13 @@ int main(int argc, char* argv[])
 				case 'm':
 					flags |= create_torrent::merkle;
 					break;
+				case 'o':
+					++i;
+					outfile = argv[i];
+					break;
+				case 'l':
+					flags |= create_torrent::symlinks;
+					break;
 				default:
 					print_usage();
 					return 1;
@@ -142,7 +162,7 @@ int main(int argc, char* argv[])
 		file_pool fp;
 		path full_path = complete(path(argv[1]));
 
-		add_files(fs, full_path, file_filter);
+		add_files(fs, full_path, file_filter, flags);
 		if (fs.num_files() == 0)
 		{
 			fputs("no files specified.\n", stderr);
@@ -173,7 +193,13 @@ int main(int argc, char* argv[])
 		// create the torrent and print it to stdout
 		std::vector<char> torrent;
 		bencode(back_inserter(torrent), t.generate());
-		fwrite(&torrent[0], 1, torrent.size(), stdout);
+		FILE* output = stdout;
+		if (!outfile.empty())
+			output = fopen(outfile.c_str(), "wb+");
+		fwrite(&torrent[0], 1, torrent.size(), output);
+
+		if (output != stdout)
+			fclose(output);
 
 #ifndef BOOST_NO_EXCEPTIONS
 	}
