@@ -41,7 +41,6 @@ POSSIBILITY OF SUCH DAMAGE.
 #include <boost/bind.hpp>
 #include <boost/ref.hpp>
 #include <boost/optional.hpp>
-#include <boost/lexical_cast.hpp>
 #include <boost/filesystem/operations.hpp>
 #include <boost/intrusive_ptr.hpp>
 #include <boost/detail/atomic_count.hpp>
@@ -55,6 +54,12 @@ POSSIBILITY OF SUCH DAMAGE.
 #include "libtorrent/udp_socket.hpp"
 #include "libtorrent/socket.hpp"
 
+namespace libtorrent
+{
+	namespace aux { struct session_impl; }
+	struct lazy_entry;
+}
+
 namespace libtorrent { namespace dht
 {
 
@@ -71,8 +76,8 @@ namespace libtorrent { namespace dht
 	{
 		friend void intrusive_ptr_add_ref(dht_tracker const*);
 		friend void intrusive_ptr_release(dht_tracker const*);
-		dht_tracker(udp_socket& sock, dht_settings const& settings
-			, entry const* state);
+		dht_tracker(libtorrent::aux::session_impl& ses, rate_limited_udp_socket& sock
+			, dht_settings const& settings, entry const* state = 0);
 
 		void start(entry const& bootstrap);
 		void stop();
@@ -84,10 +89,10 @@ namespace libtorrent { namespace dht
 		entry state() const;
 
 		void announce(sha1_hash const& ih, int listen_port
-			, boost::function<void(std::vector<tcp::endpoint> const&
-			, sha1_hash const&)> f);
+			, boost::function<void(std::vector<tcp::endpoint> const&)> f);
 
 		void dht_status(session_status& s);
+		void network_stats(int& sent, int& received);
 
 		// translate bittorrent kademlia message into the generic kademlia message
 		// used by the library
@@ -110,8 +115,11 @@ namespace libtorrent { namespace dht
 		void on_bootstrap();
 		void send_packet(msg const& m);
 
+		void incoming_error(char const* msg, lazy_entry const& e, udp::endpoint const& ep);
+
 		node_impl m_dht;
-		udp_socket& m_sock;
+		libtorrent::aux::session_impl& m_ses;
+		rate_limited_udp_socket& m_sock;
 
 		std::vector<char> m_send_buf;
 
@@ -131,11 +139,15 @@ namespace libtorrent { namespace dht
 		// used to resolve hostnames for nodes
 		udp::resolver m_host_resolver;
 
+		// sent and received bytes since queried last time
+		int m_sent_bytes;
+		int m_received_bytes;
+
 		// used to ignore abusive dht nodes
 		struct node_ban_entry
 		{
 			node_ban_entry(): count(0) {}
-			udp::endpoint src;
+			address src;
 			ptime limit;
 			int count;
 		};
@@ -157,6 +169,7 @@ namespace libtorrent { namespace dht
 		int m_failed_announces;
 
 		int m_total_message_input;
+		int m_az_message_input;
 		int m_ut_message_input;
 		int m_lt_message_input;
 		int m_mp_message_input;
