@@ -33,12 +33,11 @@ POSSIBILITY OF SUCH DAMAGE.
 #ifndef TORRENT_PEER_INFO_HPP_INCLUDED
 #define TORRENT_PEER_INFO_HPP_INCLUDED
 
-#include <vector>
-
 #include "libtorrent/socket.hpp"
 #include "libtorrent/peer_id.hpp"
 #include "libtorrent/size_type.hpp"
 #include "libtorrent/config.hpp"
+#include "libtorrent/bitfield.hpp"
 
 namespace libtorrent
 {
@@ -57,7 +56,9 @@ namespace libtorrent
 			queued = 0x100,
 			on_parole = 0x200,
 			seed = 0x400,
-			optimistic_unchoke = 0x800
+			optimistic_unchoke = 0x800,
+			snubbed = 0x1000,
+			upload_only = 0x2000
 #ifndef TORRENT_DISABLE_ENCRYPTION
 			, rc4_encrypted = 0x100000,
 			plaintext_encrypted = 0x200000
@@ -78,15 +79,29 @@ namespace libtorrent
 
 		int source;
 
+		// bw_idle: the channel is not used
+		// bw_limit: the channel is waiting for quota
+		// bw_network: the channel is waiting for an async write
+		//   for read operation to complete
+		// bw_disk: the peer is waiting for the disk io thread
+		//   to catch up
+		enum bw_state { bw_idle, bw_limit, bw_network, bw_disk };
+#ifndef TORRENT_NO_DEPRECATE
+		enum bw_state_deprecated { bw_torrent = bw_limit, bw_global = bw_limit };
+#endif
+
+		char read_state;
+		char write_state;
+		
 		tcp::endpoint ip;
-		float up_speed;
-		float down_speed;
-		float payload_up_speed;
-		float payload_down_speed;
+		int up_speed;
+		int down_speed;
+		int payload_up_speed;
+		int payload_down_speed;
 		size_type total_download;
 		size_type total_upload;
 		peer_id pid;
-		std::vector<bool> pieces;
+		bitfield pieces;
 		int upload_limit;
 		int download_limit;
 
@@ -96,11 +111,23 @@ namespace libtorrent
 		// time since last download or upload
 		time_duration last_active;
 
+		// the time until all blocks in the request
+		// queue will be d
+		time_duration download_queue_time;
+		int queue_bytes;
+
+		// the number of seconds until the current
+		// pending request times out
+		int request_timeout;
+
 		// the size of the send buffer for this peer, in bytes
 		int send_buffer_size;
 		// the number bytes that's actually used of the send buffer
 		int used_send_buffer;
 
+		int receive_buffer_size;
+		int used_receive_buffer;
+
 		// the number of failed hashes for this peer
 		int num_hashfails;
 
@@ -112,6 +139,12 @@ namespace libtorrent
 		char country[2];
 #endif
 
+#ifndef TORRENT_DISABLE_GEO_IP
+		// atonomous system this peer belongs to
+		std::string inet_as_name;
+		int inet_as;
+#endif
+
 		size_type load_balancing;
 
 		// this is the number of requests
@@ -120,6 +153,10 @@ namespace libtorrent
 		// for yet
 		int download_queue_length;
 
+		// the number of request messages
+		// waiting to be sent inside the send buffer
+		int requests_in_buffer;
+
 		// the number of requests that is
 		// tried to be maintained (this is
 		// typically a function of download speed)
@@ -147,7 +184,8 @@ namespace libtorrent
 		enum
 		{
 			standard_bittorrent = 0,
-			web_seed = 1
+			web_seed = 1,
+			http_seed = 2
 		};
 		int connection_type;
 		
@@ -157,8 +195,41 @@ namespace libtorrent
 		// number of bytes this peer has in
 		// the disk write queue
 		int pending_disk_bytes;
+
+		// numbers used for bandwidth limiting
+		int send_quota;
+		int receive_quota;
+
+		// estimated rtt to peer, in milliseconds
+		int rtt;
+
+		// the number of pieces this peer has
+		int num_pieces;
+
+		// the highest transfer rates seen for this peer
+		int download_rate_peak;
+		int upload_rate_peak;
+		
+		// the peers progress
+		float progress; // [0, 1]
+		int progress_ppm; // [0, 1000000]
 	};
 
+	struct TORRENT_EXPORT peer_list_entry
+	{
+		enum flags_t
+		{
+			banned = 1
+		};
+		
+		tcp::endpoint ip;
+		int flags;
+		boost::uint8_t failcount;
+		boost::uint8_t source;
+	};
+
+	// defined in policy.cpp
+	int source_rank(int source_bitmask);
 }
 
 #endif // TORRENT_PEER_INFO_HPP_INCLUDED
