@@ -344,7 +344,7 @@ namespace libtorrent
 		{ return m_num_uploads < m_max_uploads; }
 
 		bool choke_peer(peer_connection& c);
-		bool unchoke_peer(peer_connection& c);
+		bool unchoke_peer(peer_connection& c, bool optimistic = false);
 
 		// used by peer_connection to attach itself to a torrent
 		// since incoming connections don't know what torrent
@@ -847,7 +847,7 @@ namespace libtorrent
 		// used for tracker announces
 		deadline_timer m_tracker_timer;
 
-		void update_tracker_timer();
+		void update_tracker_timer(ptime now);
 
 		static void on_tracker_announce_disp(boost::weak_ptr<torrent> p
 			, error_code const& e);
@@ -857,6 +857,8 @@ namespace libtorrent
 		void dht_announce();
 
 #ifndef TORRENT_DISABLE_DHT
+		static void on_dht_announce_post(boost::weak_ptr<libtorrent::torrent> t
+			, std::vector<tcp::endpoint> const& peers);
 		static void on_dht_announce_response_disp(boost::weak_ptr<torrent> t
 			, std::vector<tcp::endpoint> const& peers);
 		void on_dht_announce_response(std::vector<tcp::endpoint> const& peers);
