@@ -56,7 +56,6 @@ POSSIBILITY OF SUCH DAMAGE.
 #include "libtorrent/hasher.hpp"
 #endif
 
-using boost::bind;
 using boost::shared_ptr;
 using libtorrent::aux::session_impl;
 
@@ -633,14 +632,30 @@ namespace libtorrent
 #endif
 	}
 
+	void bt_peer_connection::append_const_send_buffer(char const* buffer, int size)
+	{
+		// if we're encrypting this buffer, we need to make a copy
+		// since we'll mutate it
+#ifndef TORRENT_DISABLE_ENCRYPTION
+		if (m_encrypted && m_rc4_encrypted)
+		{
+			send_buffer(buffer, size);
+		}
+		else
+#endif
+		{
+			peer_connection::append_const_send_buffer(buffer, size);
+		}
+	}
+
 	void bt_peer_connection::send_buffer(char const* buf, int size, int flags)
 	{
 		TORRENT_ASSERT(buf);
 		TORRENT_ASSERT(size > 0);
 		
+#ifndef TORRENT_DISABLE_ENCRYPTION
 		encrypt_pending_buffer();
 
-#ifndef TORRENT_DISABLE_ENCRYPTION
 		if (m_encrypted && m_rc4_encrypted)
 		{
 			TORRENT_ASSERT(send_buffer_size() == m_encrypted_bytes);
@@ -1545,7 +1560,8 @@ namespace libtorrent
 
 		// if we're finished and this peer is uploading only
 		// disconnect it
-		if (t->is_finished() && upload_only())
+		if (t->is_finished() && upload_only()
+			&& t->settings().close_redundant_connections)
 			disconnect(errors::upload_upload_connection);
 	}
 
