@@ -51,6 +51,10 @@ POSSIBILITY OF SUCH DAMAGE.
 #include <sys/mman.h>
 #endif
 
+#ifdef TORRENT_LINUX
+#include <linux/unistd.h>
+#endif
+
 namespace libtorrent
 {
 	bool should_cancel_on_abort(disk_io_job const& j);
@@ -449,8 +453,8 @@ namespace libtorrent
 		{
 			cache_t::iterator i = std::min_element(
 				m_pieces.begin(), m_pieces.end()
-				, bind(&cached_piece_entry::last_use, _1)
-				< bind(&cached_piece_entry::last_use, _2));
+				, boost::bind(&cached_piece_entry::last_use, _1)
+				< boost::bind(&cached_piece_entry::last_use, _2));
 			if (i == m_pieces.end()) break;
 			int age = total_seconds(now - i->last_use);
 			if (age < m_settings.cache_expiry) break;
@@ -462,8 +466,8 @@ namespace libtorrent
 		{
 			cache_t::iterator i = std::min_element(
 				m_read_pieces.begin(), m_read_pieces.end()
-				, bind(&cached_piece_entry::last_use, _1)
-				< bind(&cached_piece_entry::last_use, _2));
+				, boost::bind(&cached_piece_entry::last_use, _1)
+				< boost::bind(&cached_piece_entry::last_use, _2));
 			if (i == m_read_pieces.end()) break;
 			int age = total_seconds(now - i->last_use);
 			if (age < m_settings.cache_expiry) break;
@@ -502,8 +506,8 @@ namespace libtorrent
 
 		cache_t::iterator i = std::min_element(
 			m_read_pieces.begin(), m_read_pieces.end()
-			, bind(&cached_piece_entry::last_use, _1)
-			< bind(&cached_piece_entry::last_use, _2));
+			, boost::bind(&cached_piece_entry::last_use, _1)
+			< boost::bind(&cached_piece_entry::last_use, _2));
 		if (i != m_read_pieces.end() && i != ignore)
 		{
 			// don't replace an entry that is less than one second old
@@ -627,8 +631,8 @@ namespace libtorrent
 			{
 				cache_t::iterator i = std::min_element(
 					m_pieces.begin(), m_pieces.end()
-					, bind(&cached_piece_entry::last_use, _1)
-					< bind(&cached_piece_entry::last_use, _2));
+					, boost::bind(&cached_piece_entry::last_use, _1)
+					< boost::bind(&cached_piece_entry::last_use, _2));
 				if (i == m_pieces.end()) return ret;
 				tmp = flush_and_remove(i, l);
 				blocks -= tmp;
@@ -641,8 +645,8 @@ namespace libtorrent
 			{
 				cache_t::iterator i = std::max_element(
 					m_pieces.begin(), m_pieces.end()
-					, bind(&contiguous_blocks, _1)
-					< bind(&contiguous_blocks, _2));
+					, boost::bind(&contiguous_blocks, _1)
+					< boost::bind(&contiguous_blocks, _2));
 				if (i == m_pieces.end()) return ret;
 				tmp = flush_contiguous_blocks(i, l);
 				blocks -= tmp;
@@ -1372,6 +1376,8 @@ namespace libtorrent
 		typedef std::multimap<size_type, disk_io_job> read_jobs_t;
 		read_jobs_t sorted_read_jobs;
 		read_jobs_t::iterator elevator_job_pos = sorted_read_jobs.begin();
+		size_type last_elevator_pos = 0;
+		bool need_update_elevator_pos = false;
 
 		for (;;)
 		{
@@ -1398,9 +1404,18 @@ namespace libtorrent
 				for (cache_t::iterator i = m_pieces.begin()
 					, end(m_pieces.end()); i != end; ++i)
 					flush_range(i, 0, INT_MAX, l);
+
+#ifdef TORRENT_DISABLE_POOL_ALLOCATOR
+				// since we're aborting the thread, we don't actually
+				// need to free all the blocks individually. We can just
+				// clear the piece list and the memory will be freed when we
+				// destruct the m_pool. If we're not using a pool, we actually
+				// have to free everything individually though
 				for (cache_t::iterator i = m_read_pieces.begin()
 					, end(m_read_pieces.end()); i != end; ++i)
 					free_piece(*i, l);
+#endif
+
 				m_pieces.clear();
 				m_read_pieces.clear();
 				// release the io_service to allow the run() call to return
@@ -1464,6 +1479,7 @@ namespace libtorrent
 					m_log << log_time() << " sorting_job" << std::endl;
 #endif
 					size_type phys_off = j.storage->physical_offset(j.piece, j.offset);
+					need_update_elevator_pos = need_update_elevator_pos || sorted_read_jobs.empty();
 					sorted_read_jobs.insert(std::pair<size_type, disk_io_job>(phys_off, j));
 					continue;
 				}
@@ -1477,8 +1493,16 @@ namespace libtorrent
 
 				TORRENT_ASSERT(!sorted_read_jobs.empty());
 
+				// if sorted_read_jobs used to be empty,
+				// we need to update the elevator position
+				if (need_update_elevator_pos)
+				{
+					elevator_job_pos = sorted_read_jobs.lower_bound(last_elevator_pos);
+					need_update_elevator_pos = false;
+				}
+
 				// if we've reached the end, change the elevator direction
-				if (elevator_job_pos == sorted_read_jobs.end() && elevator_direction == 1)
+				if (elevator_job_pos == sorted_read_jobs.end())
 				{
 					elevator_direction = -1;
 					--elevator_job_pos;
@@ -1489,7 +1513,7 @@ namespace libtorrent
 
 				// if we've reached the begining of the sorted list,
 				// change the elvator direction
-				if (elevator_job_pos == sorted_read_jobs.begin() && elevator_direction == -1)
+				if (elevator_job_pos == sorted_read_jobs.begin())
 					elevator_direction = 1;
 
 				// move the elevator before erasing the job we're processing
@@ -1497,6 +1521,8 @@ namespace libtorrent
 				if (elevator_direction > 0) ++elevator_job_pos;
 				else --elevator_job_pos;
 
+				TORRENT_ASSERT(to_erase != elevator_job_pos);
+				last_elevator_pos = to_erase->first;
 				sorted_read_jobs.erase(to_erase);
 			}
 
@@ -1563,6 +1589,8 @@ namespace libtorrent
 #if defined __APPLE__ && defined __MACH__ && MAC_OS_X_VERSION_MIN_REQUIRED >= 1050
 					setiopolicy_np(IOPOL_TYPE_DISK, IOPOL_SCOPE_THREAD
 						, m_settings.low_prio_disk ? IOPOL_THROTTLE : IOPOL_DEFAULT);
+#elif defined IOPRIO_WHO_PROCESS
+					syscall(ioprio_set, IOPRIO_WHO_PROCESS, getpid());
 #endif
 					break;
 				}
@@ -1603,6 +1631,7 @@ namespace libtorrent
 							continue;
 						}
 						post_callback(i->second.callback, i->second, -3);
+						if (elevator_job_pos == i) ++elevator_job_pos;
 						sorted_read_jobs.erase(i++);
 					}
 					jl.unlock();
@@ -1661,6 +1690,7 @@ namespace libtorrent
 							continue;
 						}
 						post_callback(i->second.callback, i->second, -3);
+						if (elevator_job_pos == i) ++elevator_job_pos;
 						sorted_read_jobs.erase(i++);
   					}
 					m_abort = true;
@@ -1979,7 +2009,7 @@ namespace libtorrent
 					INVARIANT_CHECK;
 
 					cache_t::iterator i = std::remove_if(
-						m_pieces.begin(), m_pieces.end(), bind(&cached_piece_entry::storage, _1) == j.storage);
+						m_pieces.begin(), m_pieces.end(), boost::bind(&cached_piece_entry::storage, _1) == j.storage);
 
 					for (cache_t::iterator k = i; k != m_pieces.end(); ++k)
 					{
@@ -2021,6 +2051,9 @@ namespace libtorrent
 					{
 						ptime now = time_now_hires();
 						TORRENT_ASSERT(now >= m_last_file_check);
+						// this happens sometimes on windows for some reason
+						if (now < m_last_file_check) now = m_last_file_check;
+
 #if BOOST_VERSION > 103600
 						if (now - m_last_file_check < milliseconds(m_settings.file_checks_delay_per_block))
 						{
