@@ -62,7 +62,8 @@ POSSIBILITY OF SUCH DAMAGE.
 #ifdef TORRENT_LINUX
 #include <sys/ioctl.h>
 #ifdef HAVE_LINUX_FIEMAP_H
-#include <linux/fiemap.h>
+#include <linux/fiemap.h> // FIEMAP_*
+#include <linux/fs.h>  // FS_IOC_FIEMAP
 #endif
 
 #include <asm/unistd.h> // For __NR_fallocate
@@ -84,6 +85,8 @@ static int my_fallocate(int fd, int mode, loff_t offset, loff_t len)
 
 #endif // TORRENT_LINUX
 
+#undef _FILE_OFFSET_BITS
+
 #include <boost/static_assert.hpp>
 // make sure the _FILE_OFFSET_BITS define worked
 // on this platform. It's supposed to make file
@@ -110,6 +113,12 @@ BOOST_STATIC_ASSERT((libtorrent::file::rw_mask & libtorrent::file::attribute_mas
 BOOST_STATIC_ASSERT((libtorrent::file::no_buffer & libtorrent::file::attribute_mask) == 0);
 #endif
 
+#ifdef TORRENT_WINDOWS
+#if defined UNICODE && !TORRENT_USE_WPATH
+#warning wide character support not available. Files will be saved using narrow string names
+#endif
+#endif // TORRENT_WINDOWS
+
 namespace libtorrent
 {
 	namespace fs = boost::filesystem;
@@ -804,7 +813,7 @@ namespace libtorrent
 
 	size_type file::phys_offset(size_type offset)
 	{
-#ifdef FS_IOC_FIEMAP
+#ifdef FIEMAP_EXTENT_UNKNOWN
 		// for documentation of this feature
 		// http://lwn.net/Articles/297696/
 		struct
@@ -909,7 +918,9 @@ namespace libtorrent
 		{
 			// only allocate the space if the file
 			// is not fully allocated
-			offs.LowPart = GetCompressedFileSize(m_path.c_str(), &offs.HighPart);
+			DWORD high_dword = 0;
+			offs.LowPart = GetCompressedFileSize(m_path.c_str(), &high_dword);
+			offs.HighPart = high_dword;
 			ec.assign(GetLastError(), get_system_category());
 			if (ec) return false;
 			if (offs.QuadPart != s)
@@ -920,12 +931,7 @@ namespace libtorrent
 				SetFileValidData(m_file_handle, offs.QuadPart);
 			}
 		}
-#endif
-		if (::SetEndOfFile(m_file_handle) == FALSE)
-		{
-			ec = error_code(GetLastError(), get_system_category());
-			return false;
-		}
+#endif // _WIN32_WINNT >= 0x501
 #else
 		struct stat st;
 		if (fstat(m_fd, &st) != 0)
@@ -961,8 +967,11 @@ namespace libtorrent
 				ec = error_code(errno, get_posix_category());
 				return false;
 			}
-#elif defined TORRENT_LINUX
-			int ret = my_fallocate(m_fd, 0, 0, s);
+#endif // F_PREALLOCATE
+
+			int ret;
+#if defined TORRENT_LINUX
+			ret = my_fallocate(m_fd, 0, 0, s);
 			// if we return 0, everything went fine
 			// the fallocate call succeeded
 			if (ret == 0) return true;
@@ -976,24 +985,22 @@ namespace libtorrent
 				ec.assign(ret, get_posix_category());
 				return false;
 			}
+#endif // TORRENT_LINUX
+
+#if TORRENT_HAS_FALLOCATE
 			// if fallocate failed, we have to use posix_fallocate
 			// which can be painfully slow
+			// if you get a compile error here, you might want to
+			// define TORRENT_HAS_FALLOCATE to 0.
 			ret = posix_fallocate(m_fd, 0, s);
 			if (ret != 0)
 			{
 				ec = error_code(ret, get_posix_category());
 				return false;
 			}
-#elif TORRENT_HAS_FALLOCATE
-			int ret = posix_fallocate(m_fd, 0, s);
-			if (ret != 0)
-			{
-				ec = error_code(ret, get_posix_category());
-				return false;
-			}
-#endif
+#endif // TORRENT_HAS_FALLOCATE
 		}
-#endif
+#endif // TORRENT_WINDOWS
 		return true;
 	}
 
