@@ -60,7 +60,6 @@ POSSIBILITY OF SUCH DAMAGE.
 #include "libtorrent/aux_/session_impl.hpp"
 
 using namespace libtorrent;
-using boost::bind;
 
 namespace libtorrent
 {
@@ -216,13 +215,20 @@ namespace libtorrent
 				++i;
 		}
 
+#if defined(TORRENT_VERBOSE_LOGGING) || defined(TORRENT_LOGGING)
+		boost::shared_ptr<request_callback> cb = requester();
+		if (cb)
+		{
+			cb->debug_log("*** TRACKER_FILTER");
+		}
+#endif
 		if (endpoints.empty())
 			fail(-1, "blocked by IP filter");
 	}
 
 	void http_tracker_connection::on_connect(http_connection& c)
 	{
-    	error_code ec;
+		error_code ec;
 		tcp::endpoint ep = c.socket().remote_endpoint(ec);
 		m_tracker_ip = ep.address();
 		boost::shared_ptr<request_callback> cb = requester();
@@ -437,7 +443,9 @@ namespace libtorrent
 		lazy_entry const* ipv6_peers = 0;
 #endif
 
-		if (peers_ent == 0 && ipv6_peers == 0)
+		// if we didn't receive any peers. We don't care if we're stopping anyway
+		if (peers_ent == 0 && ipv6_peers == 0
+			&& tracker_req().event != tracker_request::stopped)
 		{
 			fail(-1, "missing 'peers' and 'peers6' entry in tracker response"
 				, interval, min_interval);
