@@ -53,7 +53,6 @@ POSSIBILITY OF SUCH DAMAGE.
 #include "libtorrent/assert.hpp"
 #include "libtorrent/enum_net.hpp"
 
-using boost::bind;
 using namespace libtorrent;
 
 natpmp::natpmp(io_service& ios, address const& listen_interface
@@ -112,7 +111,7 @@ void natpmp::rebind(address const& listen_interface)
 	}
 
 	m_socket.async_receive_from(asio::buffer(&m_response_buffer, 16)
-		, m_remote, bind(&natpmp::on_reply, self(), _1, _2));
+		, m_remote, boost::bind(&natpmp::on_reply, self(), _1, _2));
 
 	for (std::vector<mapping_t>::iterator i = m_mappings.begin()
 		, end(m_mappings.end()); i != end; ++i)
@@ -319,6 +318,7 @@ void natpmp::send_map_request(int i, mutex_t::scoped_lock& l)
 	error_code ec;
 	m_socket.send_to(asio::buffer(buf, 12), m_nat_endpoint, 0, ec);
 	m.map_sent = true;
+	m.outstanding_request = true;
 	if (m_abort)
 	{
 		// when we're shutting down, ignore the
@@ -333,7 +333,7 @@ void natpmp::send_map_request(int i, mutex_t::scoped_lock& l)
 		// linear back-off instead of exponential
 		++m_retry_count;
 		m_send_timer.expires_from_now(milliseconds(250 * m_retry_count), ec);
-		m_send_timer.async_wait(bind(&natpmp::resend_request, self(), i, _1));
+		m_send_timer.async_wait(boost::bind(&natpmp::resend_request, self(), i, _1));
 	}
 }
 
@@ -372,7 +372,7 @@ void natpmp::on_reply(error_code const& e
 	}
 
 	m_socket.async_receive_from(asio::buffer(&m_response_buffer, 16)
-		, m_remote, bind(&natpmp::on_reply, self(), _1, _2));
+		, m_remote, boost::bind(&natpmp::on_reply, self(), _1, _2));
 
 	// simulate packet loss
 /*
@@ -428,6 +428,7 @@ void natpmp::on_reply(error_code const& e
 		if (private_port != i->local_port) continue;
 		if (protocol != i->protocol) continue;
 		if (!i->map_sent) continue;
+		if (!i->outstanding_request) continue;
 		m = &*i;
 		index = i - m_mappings.begin();
 		break;
@@ -439,6 +440,8 @@ void natpmp::on_reply(error_code const& e
 		log(msg, l);
 		return;
 	}
+	m->outstanding_request = false;
+
 	log(msg, l);
 
 	if (public_port == 0 || lifetime == 0)
@@ -483,15 +486,15 @@ void natpmp::on_reply(error_code const& e
 	m_currently_mapping = -1;
 	m->action = mapping_t::action_none;
 	m_send_timer.cancel(ec);
-	update_expiration_timer();
+	update_expiration_timer(l);
 	try_next_mapping(index, l);
 }
 
-void natpmp::update_expiration_timer()
+void natpmp::update_expiration_timer(boost::mutex::scoped_lock& l)
 {
 	if (m_abort) return;
 
-	ptime now = time_now();
+	ptime now = time_now() + milliseconds(100);
 /*
 #if defined(TORRENT_LOGGING) || defined(TORRENT_VERBOSE_LOGGING)
 	m_log << time_now_string() << "update_expiration_timer " << std::endl;
@@ -515,10 +518,20 @@ void natpmp::update_expiration_timer()
 	{
 		if (i->protocol == none
 			|| i->action != mapping_t::action_none) continue;
-		if (i->expires < min_expire)
+		int index = i - m_mappings.begin();
+		if (i->expires < now)
+		{
+			char msg[200];
+			snprintf(msg, sizeof(msg), "mapping %u expired", index);
+			log(msg, l);
+			i->action = mapping_t::action_add;
+			if (m_next_refresh == index) m_next_refresh = -1;
+			update_mapping(index, l);
+		}
+		else if (i->expires < min_expire)
 		{
 			min_expire = i->expires;
-			min_index = i - m_mappings.begin();
+			min_index = index;
 		}
 	}
 
@@ -538,7 +551,7 @@ void natpmp::update_expiration_timer()
 		error_code ec;
 		if (m_next_refresh >= 0) m_refresh_timer.cancel(ec);
 		m_refresh_timer.expires_from_now(min_expire - now, ec);
-		m_refresh_timer.async_wait(bind(&natpmp::mapping_expired, self(), _1, min_index));
+		m_refresh_timer.async_wait(boost::bind(&natpmp::mapping_expired, self(), _1, min_index));
 		m_next_refresh = min_index;
 	}
 }
