@@ -64,8 +64,6 @@ namespace libtorrent
 	class peer_connection;
 }
 
-using boost::bind;
-
 namespace
 {
 	using namespace libtorrent;
@@ -135,6 +133,7 @@ namespace libtorrent
 		if (t.is_seed()) return;
 		if (c.no_download()) return;
 		if (t.upload_mode()) return;
+		if (c.is_disconnecting()) return;
 
 		// don't request pieces before we have the metadata
 		if (!t.valid_metadata()) return;
@@ -295,7 +294,7 @@ namespace libtorrent
 		// find the block with the fewest requests to it
 		std::vector<piece_block>::iterator i = std::min_element(
 			busy_pieces.begin(), busy_pieces.end()
-			, bind(&piece_picker::num_peers, boost::cref(p), _1) <
+			, boost::bind(&piece_picker::num_peers, boost::cref(p), _1) <
 			bind(&piece_picker::num_peers, boost::cref(p), _2));
 #ifdef TORRENT_DEBUG
 		piece_picker::downloading_piece st;
@@ -308,7 +307,7 @@ namespace libtorrent
 		ptime last_request = p.last_request(i->piece_index);
 		ptime now = time_now();
 
-		// don't re-request from a piece more often than once every 20 seconds
+		// don't re-request from a piece more often than once every 5 seconds
 		// TODO: make configurable
 		if (now - last_request < seconds(5))
 			return;
@@ -377,6 +376,7 @@ namespace libtorrent
 	void policy::erase_peer(iterator i)
 	{
 		INVARIANT_CHECK;
+		TORRENT_ASSERT(i != m_peers.end());
 
 		if (m_torrent->has_picker())
 			m_torrent->picker().clear_peer(*i);
@@ -388,15 +388,30 @@ namespace libtorrent
 		}
 		TORRENT_ASSERT(m_num_connect_candidates < m_peers.size());
 		if (m_round_robin > i - m_peers.begin()) --m_round_robin;
+		if (m_round_robin >= m_peers.size()) m_round_robin = 0;
+
+#ifdef TORRENT_DEBUG
+		TORRENT_ASSERT((*i)->in_use);
+		(*i)->in_use = false;
+#endif
 
 #if TORRENT_USE_IPV6
 		if ((*i)->is_v6_addr)
+		{
+			TORRENT_ASSERT(m_torrent->session().m_ipv6_peer_pool.is_from(
+				static_cast<ipv6_peer*>(*i)));
 			m_torrent->session().m_ipv6_peer_pool.destroy(
 				static_cast<ipv6_peer*>(*i));
+		}
 		else
 #endif
+		{
+			TORRENT_ASSERT(m_torrent->session().m_ipv4_peer_pool.is_from(
+				static_cast<ipv4_peer*>(*i)));
 			m_torrent->session().m_ipv4_peer_pool.destroy(
 				static_cast<ipv4_peer*>(*i));
+		}
+
 		m_peers.erase(i);
 	}
 
@@ -420,8 +435,8 @@ namespace libtorrent
 		INVARIANT_CHECK;
 
 		int max_peerlist_size = m_torrent->is_paused()
-			?m_torrent->settings().max_paused_peerlist_size
-			:m_torrent->settings().max_peerlist_size;
+			? m_torrent->settings().max_paused_peerlist_size
+			: m_torrent->settings().max_peerlist_size;
 
 		if (max_peerlist_size == 0 || m_peers.empty()) return;
 
@@ -450,6 +465,8 @@ namespace libtorrent
 					if (should_erase_immediately(pe))
 					{
 						if (erase_candidate > current) --erase_candidate;
+						TORRENT_ASSERT(current >= 0 && current < m_peers.size());
+						--round_robin;
 						erase_peer(m_peers.begin() + current);
 					}
 					else
@@ -463,7 +480,10 @@ namespace libtorrent
 		}
 		
 		if (erase_candidate > -1)
+		{
+			TORRENT_ASSERT(erase_candidate >= 0 && erase_candidate < m_peers.size());
 			erase_peer(m_peers.begin() + erase_candidate);
+		}
 	}
 
 	void policy::ban_peer(policy::peer* p)
@@ -538,7 +558,7 @@ namespace libtorrent
 			external_ip = address_v4(bytes);
 		}
 
-		if (m_round_robin == m_peers.size()) m_round_robin = 0;
+		if (m_round_robin >= m_peers.size()) m_round_robin = 0;
 
 #ifndef TORRENT_DISABLE_DHT
 		bool pinged = false;
@@ -551,7 +571,7 @@ namespace libtorrent
 		for (int iterations = (std::min)(int(m_peers.size()), 300);
 			iterations > 0; --iterations)
 		{
-			if (m_round_robin == m_peers.size()) m_round_robin = 0;
+			if (m_round_robin >= m_peers.size()) m_round_robin = 0;
 
 			peer& pe = *m_peers[m_round_robin];
 			int current = m_round_robin;
@@ -583,6 +603,7 @@ namespace libtorrent
 					{
 						if (erase_candidate > current) --erase_candidate;
 						if (candidate > current) --candidate;
+						--m_round_robin;
 						erase_peer(m_peers.begin() + current);
 					}
 					else
@@ -778,7 +799,6 @@ namespace libtorrent
 				return false;
 			}
 
-			if (m_round_robin > iter - m_peers.begin()) ++m_round_robin;
 #if TORRENT_USE_IPV6
 			bool is_v6 = c.remote().address().is_v6();
 #endif
@@ -788,6 +808,9 @@ namespace libtorrent
 #endif
 				(peer*)m_torrent->session().m_ipv4_peer_pool.malloc();
 			if (p == 0) return false;
+
+//			TORRENT_ASSERT(p->in_use == false);
+
 #if TORRENT_USE_IPV6
 			if (is_v6)
 				m_torrent->session().m_ipv6_peer_pool.set_next_size(500);
@@ -802,8 +825,14 @@ namespace libtorrent
 #endif
 				new (p) ipv4_peer(c.remote(), false, 0);
 
+#ifdef TORRENT_DEBUG
+			p->in_use = true;
+#endif
+
 			iter = m_peers.insert(iter, p);
 
+			if (m_round_robin >= iter - m_peers.begin()) ++m_round_robin;
+
 			i = *iter;
 #ifndef TORRENT_DISABLE_GEO_IP
 			int as = ses.as_for_ip(c.remote().address());
@@ -1001,8 +1030,6 @@ namespace libtorrent
 				);
 			}
 
-			if (m_round_robin > iter - m_peers.begin()) ++m_round_robin;
-
 			// we don't have any info about this peer.
 			// add a new entry
 #if TORRENT_USE_IPV6
@@ -1028,8 +1055,14 @@ namespace libtorrent
 #endif
 				new (p) ipv4_peer(remote, true, src);
 
+#ifdef TORRENT_DEBUG
+			p->in_use = true;
+#endif
+
 			iter = m_peers.insert(iter, p);
 
+			if (m_round_robin >= iter - m_peers.begin()) ++m_round_robin;
+
 			i = *iter;
 #ifndef TORRENT_DISABLE_ENCRYPTION
 			if (flags & 0x01) i->pe_support = true;
