@@ -104,7 +104,6 @@ POSSIBILITY OF SUCH DAMAGE.
 #include "libtorrent/escape_string.hpp"
 
 namespace fs = boost::filesystem;
-using boost::bind;
 
 #if defined TORRENT_DEBUG && defined TORRENT_STORAGE_DEBUG && TORRENT_USE_IOSTREAM
 namespace
@@ -2470,6 +2469,7 @@ ret:
 		}
 
 		TORRENT_ASSERT(m_state == state_full_check);
+		if (m_state == state_finished) return 0;
 
 		int skip = check_one_piece(have_piece);
 		TORRENT_ASSERT(m_current_slot <= m_files.num_pieces());
@@ -2605,8 +2605,10 @@ ret:
 		{
 			if (m_storage->error()
 #ifdef TORRENT_WINDOWS
+				&& m_storage->error() != error_code(ERROR_PATH_NOT_FOUND, get_system_category())
 				&& m_storage->error() != error_code(ERROR_FILE_NOT_FOUND, get_system_category())
-				&& m_storage->error() != error_code(ERROR_HANDLE_EOF, get_system_category()))
+				&& m_storage->error() != error_code(ERROR_HANDLE_EOF, get_system_category())
+				&& m_storage->error() != error_code(ERROR_INVALID_HANDLE, get_system_category()))
 #else
 				&& m_storage->error() != error_code(ENOENT, get_posix_category()))
 #endif
