@@ -391,6 +391,18 @@ namespace libtorrent
 	int announce_entry::min_announce_in() const
 	{ return total_seconds(time_now() - min_announce); }
 
+	bool announce_entry::can_announce(ptime now, bool is_seed) const
+	{
+		// if we're a seed and we haven't sent a completed
+		// event, we need to let this announce through
+		bool need_send_complete = is_seed && !complete_sent;
+
+		return now >= next_announce
+			&& (now >= min_announce || need_send_complete)
+			&& (fails < fail_limit || fail_limit == 0)
+			&& !updating;
+	}
+
 	torrent_info::torrent_info(torrent_info const& t)
 		: m_files(t.m_files)
 		, m_orig_files(t.m_orig_files)
@@ -1037,9 +1049,8 @@ namespace libtorrent
 		e.source = announce_entry::source_client;
 		m_urls.push_back(e);
 
-		using boost::bind;
-		std::sort(m_urls.begin(), m_urls.end(), bind(&announce_entry::tier, _1)
-			< bind(&announce_entry::tier, _2));
+		std::sort(m_urls.begin(), m_urls.end(), boost::bind(&announce_entry::tier, _1)
+			< boost::bind(&announce_entry::tier, _2));
 	}
 
 #if !defined TORRENT_NO_DEPRECATE && TORRENT_USE_IOSTREAM
