@@ -54,8 +54,6 @@ POSSIBILITY OF SUCH DAMAGE.
 #include "libtorrent/aux_/session_impl.hpp"
 #include "libtorrent/escape_string.hpp"
 
-using boost::bind;
-
 namespace libtorrent
 {
 
@@ -97,7 +95,9 @@ namespace libtorrent
 
 		if (ec)
 		{
-			fail(-1, ec.message().c_str());
+			// never call fail() when the session mutex is locked!
+			m_socket.get_io_service().post(boost::bind(
+				&tracker_connection::fail_disp, self(), -1, ec.message()));
 			return;
 		}
 		
@@ -151,7 +151,12 @@ namespace libtorrent
 			i != m_endpoints.end();)
 		{
 			if (m_ses.m_ip_filter.access(i->address()) == ip_filter::blocked) 
+			{
+#if defined TORRENT_VERBOSE_LOGGING || defined TORRENT_LOGGING
+				if (cb) cb->debug_log("*** UDP_TRACKER [ IP blocked by filter: " + print_address(i->address()) + " ]");
+#endif
 				i = m_endpoints.erase(i);
+			}
 			else
 				++i;
 		}
@@ -493,8 +498,6 @@ namespace libtorrent
 
 	void udp_tracker_connection::on_scrape_response(char const* buf, int size)
 	{
-		buf += 8; // skip header
-
 		restart_read_timeout();
 		int action = detail::read_int32(buf);
 		int transaction = detail::read_int32(buf);
