@@ -89,15 +89,18 @@ bool print_alerts(libtorrent::session& ses, char const* name
 		}
 		TEST_CHECK(alert_cast<fastresume_rejected_alert>(a.get()) == 0 || allow_failed_fastresume);
 
-		TEST_CHECK(alert_cast<peer_error_alert>(a.get()) == 0
+		peer_error_alert* pea = alert_cast<peer_error_alert>(a.get());
+		TEST_CHECK(pea == 0
 			|| (!handles.empty() && h.is_seed())
-			|| a->message() == "connecting to peer"
-			|| a->message() == "closing connection to ourself"
-			|| a->message() == "duplicate connection"
-			|| a->message() == "duplicate peer-id, connection closed"
-			|| (allow_disconnects && a->message() == "Broken pipe")
-			|| (allow_disconnects && a->message() == "Connection reset by peer")
-			|| (allow_disconnects && a->message() == "End of file."));
+			|| pea->error.message() == "connecting to peer"
+			|| pea->error.message() == "connected to ourselves"
+			|| pea->error.message() == "duplicate connection"
+			|| pea->error.message() == "duplicate peer-id"
+			|| pea->error.message() == "upload to upload connection"
+			|| pea->error.message() == "stopping torrent"
+			|| (allow_disconnects && pea->error.message() == "Broken pipe")
+			|| (allow_disconnects && pea->error.message() == "Connection reset by peer")
+			|| (allow_disconnects && pea->error.message() == "End of file."));
 		a = ses.pop_alert();
 	}
 	return ret;
@@ -353,7 +356,7 @@ void stop_web_server()
 {
 	if (web_server && web_ios)
 	{
-		web_ios->stop();
+		web_ios->post(boost::bind(&io_service::stop, web_ios));
 		web_server->join();
 		web_server.reset();
 		web_ios = 0;
@@ -464,25 +467,33 @@ void web_server_thread(int* port, bool ssl)
 	char buf[10000];
 	int len = 0;
 	int offset = 0;
+	bool connection_close = false;
 	stream_socket s(ios);
 
 	for (;;)
 	{
-		s.close(ec);
-
-		len = 0;
-		offset = 0;
-		accept_done = false;
-		acceptor.async_accept(s, &on_accept);
-		ios.reset();
-		ios.run_one();
-		if (!accept_done)
+		if (connection_close)
 		{
-			fprintf(stderr, "accept failed\n");
-			return;
+			s.close(ec);
+			connection_close = false;
 		}
 
-		if (!s.is_open()) continue;
+		if (!s.is_open())
+		{
+			len = 0;
+			offset = 0;
+
+			accept_done = false;
+			acceptor.async_accept(s, &on_accept);
+			ios.reset();
+			ios.run_one();
+			if (!accept_done)
+			{
+				fprintf(stderr, "accept failed\n");
+				return;
+			}
+			if (!s.is_open()) continue;
+		}
 
 		http_parser p;
 		bool failed = false;
@@ -514,9 +525,9 @@ void web_server_thread(int* port, bool ssl)
 					break;
 				}
 				len += received;
-	
-	
+		
 				p.incoming(buffer::const_interval(buf + offset, buf + len), error);
+
 				TEST_CHECK(error == false);
 				if (error)
 				{
@@ -525,11 +536,27 @@ void web_server_thread(int* port, bool ssl)
 					break;
 				}
 			}
+
+			std::string connection = p.header("connection");
+			std::string via = p.header("via");
+
+			// The delegate proxy doesn't say connection close, but it expects it to be closed
+			// the Via: header is an indicator of delegate making the request
+			if (connection == "close" || !via.empty())
+			{
+				connection_close = true;
+			}
+
 //			fprintf(stderr, "%s", std::string(buf + offset, p.body_start()).c_str());
 
-			if (failed) break;
+			if (failed)
+			{
+				s.close(ec);
+				break;
+			}
 
 			offset += p.body_start() + p.content_length();
+//			fprintf(stderr, "offset: %d len: %d\n", offset, len);
 
 			if (p.method() != "get" && p.method() != "post")
 			{
@@ -598,14 +625,18 @@ void web_server_thread(int* port, bool ssl)
 					write(s, boost::asio::buffer(&file_buf[0] + start, end - start + 1)
 						, boost::asio::transfer_all(), ec);
 				}
-				fprintf(stderr, "send %d bytes of payload\n", end - start + 1);
+//				fprintf(stderr, "send %d bytes of payload\n", end - start + 1);
 			}
 			else
 			{
 				send_response(s, ec, 200, "OK", extra_header, file_buf.size());
-				write(s, boost::asio::buffer(&file_buf[0], file_buf.size()), boost::asio::transfer_all(), ec);
+				if (!file_buf.empty())
+					write(s, boost::asio::buffer(&file_buf[0], file_buf.size()), boost::asio::transfer_all(), ec);
 			}
 //			fprintf(stderr, "%d bytes left in receive buffer. offset: %d\n", len - offset, offset);
+			memmove(buf, buf + offset, len - offset);
+			len -= offset;
+			offset = 0;
 		} while (offset < len);
 	}
 	fprintf(stderr, "exiting web server thread\n");
