@@ -58,6 +58,12 @@ char* piece0 = page_aligned_allocator::malloc(piece_size);
 char* piece1 = page_aligned_allocator::malloc(piece_size);
 char* piece2 = page_aligned_allocator::malloc(piece_size);
 
+void signal_bool(bool* b, char const* string)
+{
+	*b = true;
+	std::cerr << string << std::endl;
+}
+
 void on_read_piece(int ret, disk_io_job const& j, char const* data, int size)
 {
 	std::cerr << "on_read_piece piece: " << j.piece << std::endl;
@@ -70,11 +76,19 @@ void on_check_resume_data(int ret, disk_io_job const& j, bool* done)
 	std::cerr << "on_check_resume_data ret: " << ret;
 	switch (ret)
 	{
-		case 0: std::cerr << " success" << std::endl; break;
-		case -1: std::cerr << " need full check" << std::endl; break;
-		case -2: std::cerr << " disk error: " << j.str
-			<< " file: " << j.error_file << std::endl; break;
-		case -3: std::cerr << " aborted" << std::endl; break;
+		case piece_manager::no_error:
+			std::cerr << " success" << std::endl;
+			break;
+		case piece_manager::fatal_disk_error:
+			std::cerr << " disk error: " << j.str
+				<< " file: " << j.error_file << std::endl;
+			break;
+		case piece_manager::need_full_check:
+			std::cerr << " need full check" << std::endl;
+			break;
+		case piece_manager::disk_check_aborted:
+			std::cerr << " aborted" << std::endl;
+			break;
 	}
 	*done = true;
 }
@@ -85,11 +99,23 @@ void on_check_files(int ret, disk_io_job const& j, bool* done)
 
 	switch (ret)
 	{
-		case 0: std::cerr << " done" << std::endl; *done = true; break;
-		case -1: std::cerr << " current slot: " << j.piece << " have: " << j.offset << std::endl; break;
-		case -2: std::cerr << " disk error: " << j.str
-			<< " file: " << j.error_file << std::endl; *done = true; break;
-		case -3: std::cerr << " aborted" << std::endl; *done = true; break;
+		case piece_manager::no_error:
+			std::cerr << " done" << std::endl;
+			*done = true;
+			break;
+		case piece_manager::fatal_disk_error:
+			std::cerr << " disk error: " << j.str
+				<< " file: " << j.error_file << std::endl;
+			*done = true;
+			break;
+		case piece_manager::need_full_check:
+			std::cerr << " current slot: " << j.piece
+				<< " have: " << j.offset << std::endl;
+			break;
+		case piece_manager::disk_check_aborted:
+			std::cerr << " aborted" << std::endl;
+			*done = true;
+			break;
 	}
 }
 
@@ -106,11 +132,12 @@ void on_read(int ret, disk_io_job const& j, bool* done)
 	}
 }
 
-void on_move_storage(int ret, disk_io_job const& j, std::string path)
+void on_move_storage(int ret, bool* done, disk_io_job const& j, std::string path)
 {
 	std::cerr << "on_move_storage ret: " << ret << " path: " << j.str << std::endl;
 	TEST_CHECK(ret == 0);
 	TEST_CHECK(j.str == path);
+	*done = true;
 }
 
 void print_error(int ret, boost::scoped_ptr<storage_interface> const& s)
@@ -126,11 +153,12 @@ int bufs_size(file::iovec_t const* bufs, int num_bufs);
 // simulate a very slow first read
 struct test_storage : storage_interface
 {
-	test_storage() {}
+	test_storage(): m_started(false), m_ready(false) {}
 
 	virtual bool initialize(bool allocate_files) { return true; }
 	virtual bool has_any_file() { return true; }
 
+
 	int write(
 		const char* buf
 		, int slot
@@ -148,8 +176,16 @@ struct test_storage : storage_interface
 	{
 		if (slot == 0 || slot == 5999)
 		{
-			boost::thread::sleep(boost::get_system_time()
-				+ boost::posix_time::seconds(2));
+			boost::mutex::scoped_lock l(m_mutex);
+			std::cerr << "--- starting job " << slot << " waiting for main thread ---\n" << std::endl;
+			m_ready = true;
+			m_ready_condition.notify_all();
+
+			while (!m_started)
+				m_condition.wait(l);
+
+			m_ready = false;
+			m_started = false;
 			std::cerr << "--- starting ---\n" << std::endl;
 		}
 		return size;
@@ -187,6 +223,28 @@ struct test_storage : storage_interface
 	virtual bool delete_files() { return false; }
 
 	virtual ~test_storage() {}
+
+	void wait_for_ready()
+	{
+		boost::mutex::scoped_lock l(m_mutex);
+		while (!m_ready)
+			m_ready_condition.wait(l);
+	}
+
+	void start()
+	{
+		boost::mutex::scoped_lock l(m_mutex);
+		m_started = true;
+		m_condition.notify_all();
+	}
+
+private:
+	boost::condition_variable m_ready_condition;
+	boost::condition_variable m_condition;
+	boost::mutex m_mutex;
+	bool m_started;
+	bool m_ready;
+
 };
 
 storage_interface* create_test_storage(file_storage const& fs
@@ -261,12 +319,12 @@ void run_elevator_test()
 		dio.add_job(j);
 
 		// test the elevator going up
+		turns = 0;
 		direction = 1;
 		last_job = 0;
 		add_job(dio, 0, pm); // trigger delay in storage
 		// make sure the job is processed
-		boost::thread::sleep(boost::get_system_time()
-			+ boost::posix_time::milliseconds(200));
+		((test_storage*)pm->get_storage_impl())->wait_for_ready();
 
 		boost::uint32_t p = 1234513;
 		for (int i = 0; i < 100; ++i)
@@ -277,20 +335,22 @@ void run_elevator_test()
 			add_job(dio, job, pm);
 		}
 
+		((test_storage*)pm->get_storage_impl())->start();
+
 		for (int i = 0; i < 101; ++i)
 			ios.run_one();
 
-		TEST_CHECK(turns < 2);
+		TEST_CHECK(turns == 0);
 		TEST_CHECK(job_counter == 0);
 		std::cerr << "number of elevator turns: " << turns << std::endl;
 
 		// test the elevator going down
+		turns = 0;
 		direction = -1;
 		last_job = 6000;
 		add_job(dio, 5999, pm); // trigger delay in storage
 		// make sure the job is processed
-		boost::thread::sleep(boost::get_system_time()
-			+ boost::posix_time::milliseconds(200));
+		((test_storage*)pm->get_storage_impl())->wait_for_ready();
 
 		for (int i = 0; i < 100; ++i)
 		{
@@ -300,10 +360,12 @@ void run_elevator_test()
 			add_job(dio, job, pm);
 		}
 
+		((test_storage*)pm->get_storage_impl())->start();
+
 		for (int i = 0; i < 101; ++i)
 			ios.run_one();
 
-		TEST_CHECK(turns < 2);
+		TEST_CHECK(turns == 0);
 		TEST_CHECK(job_counter == 0);
 		std::cerr << "number of elevator turns: " << turns << std::endl;
 
@@ -317,8 +379,7 @@ void run_elevator_test()
 		direction = 0;
 		add_job(dio, 0, pm); // trigger delay in storage
 		// make sure the job is processed
-		boost::thread::sleep(boost::get_system_time()
-			+ boost::posix_time::milliseconds(200));
+		((test_storage*)pm->get_storage_impl())->wait_for_ready();
 
 		for (int i = 0; i < 100; ++i)
 		{
@@ -328,6 +389,8 @@ void run_elevator_test()
 			add_job(dio, job, pm);
 		}
 
+		((test_storage*)pm->get_storage_impl())->start();
+
 		for (int i = 0; i < 101; ++i)
 			ios.run_one();
 
@@ -341,6 +404,22 @@ void run_elevator_test()
 	}
 }
 
+void run_until(io_service& ios, bool const& done)
+{
+	while (!done)
+	{
+		ios.reset();
+		error_code ec;
+		ios.run_one(ec);
+		if (ec)
+		{
+			std::cerr << "run_one: " << ec.message() << std::endl;
+			return;
+		}
+		std::cerr << "done: " << done << std::endl;
+	}
+}
+
 void run_storage_tests(boost::intrusive_ptr<torrent_info> info
 	, file_storage& fs
 	, path const& test_path
@@ -426,19 +505,11 @@ void run_storage_tests(boost::intrusive_ptr<torrent_info> info
 	lazy_entry frd;
 	pm->async_check_fastresume(&frd, boost::bind(&on_check_resume_data, _1, _2, &done));
 	ios.reset();
-	while (!done)
-	{
-		ios.reset();
-		ios.run_one(ec);
-	}
+	run_until(ios, done);
 
 	done = false;
 	pm->async_check_files(boost::bind(&on_check_files, _1, _2, &done));
-	while (!done)
-	{
-		ios.reset();
-		ios.run_one(ec);
-	}
+	run_until(ios, done);
 
 	done = false;
 	peer_request r;
@@ -446,22 +517,16 @@ void run_storage_tests(boost::intrusive_ptr<torrent_info> info
 	r.start = 10;
 	r.length = 16 * 1024;
 	pm->async_read(r, boost::bind(&on_read, _1, _2, &done));
-	while (!done)
-	{
-		ios.reset();
-		ios.run_one(ec);
-	}
+	run_until(ios, done);
 
 	// test rename_file
 	remove(test_path / "part0");
 	TEST_CHECK(exists(test_path / "temp_storage/test1.tmp"));
 	TEST_CHECK(!exists(test_path / "part0"));	
 	boost::function<void(int, disk_io_job const&)> none;
-	pm->async_rename_file(0, "part0", none);
-
-	test_sleep(1000);
-	ios.reset();
-	ios.poll(ec);
+	done = false;
+	pm->async_rename_file(0, "part0", boost::bind(&signal_bool, &done, "rename_file"));
+	run_until(ios, done);
 
 	TEST_CHECK(!exists(test_path / "temp_storage/test1.tmp"));
 	TEST_CHECK(!exists(test_path / "temp_storage2"));
@@ -469,11 +534,11 @@ void run_storage_tests(boost::intrusive_ptr<torrent_info> info
 
 	// test move_storage with two files in the root directory
 	TEST_CHECK(exists(test_path / "temp_storage"));
-	pm->async_move_storage(test_path / "temp_storage2", bind(on_move_storage, _1, _2, (test_path / "temp_storage2").string()));
 
-	test_sleep(2000);
-	ios.reset();
-	ios.poll(ec);
+	done = false;
+	pm->async_move_storage(test_path / "temp_storage2"
+		, boost::bind(on_move_storage, _1, &done, _2, (test_path / "temp_storage2").string()));
+	run_until(ios, done);
 
 	if (fs.num_files() > 1)
 	{
@@ -482,11 +547,9 @@ void run_storage_tests(boost::intrusive_ptr<torrent_info> info
 	}
 	TEST_CHECK(exists(test_path / "temp_storage2/part0"));	
 
-	pm->async_move_storage(test_path, bind(on_move_storage, _1, _2, test_path.string()));
-
-	test_sleep(2000);
-	ios.reset();
-	ios.poll(ec);
+	done = false;
+	pm->async_move_storage(test_path, boost::bind(on_move_storage, _1, &done, _2, test_path.string()));
+	run_until(ios, done);
 
 	TEST_CHECK(exists(test_path / "part0"));	
 	TEST_CHECK(!exists(test_path / "temp_storage2/temp_storage"));	
@@ -495,18 +558,22 @@ void run_storage_tests(boost::intrusive_ptr<torrent_info> info
 	r.piece = 0;
 	r.start = 0;
 	r.length = block_size;
-	pm->async_read(r, bind(&on_read_piece, _1, _2, piece0, block_size));
+	pm->async_read(r, boost::bind(&on_read_piece, _1, _2, piece0, block_size));
 	r.piece = 1;
-	pm->async_read(r, bind(&on_read_piece, _1, _2, piece1, block_size));
+	pm->async_read(r, boost::bind(&on_read_piece, _1, _2, piece1, block_size));
 	r.piece = 2;
-	pm->async_read(r, bind(&on_read_piece, _1, _2, piece2, block_size));
-	pm->async_release_files(none);
+	pm->async_read(r, boost::bind(&on_read_piece, _1, _2, piece2, block_size));
 
-	pm->async_rename_file(0, "temp_storage/test1.tmp", none);
-	test_sleep(2000);
-	ios.reset();
-	ios.poll(ec);
+	std::cerr << "async_release_files" << std::endl;
+	done = false;
+	pm->async_release_files(boost::bind(&signal_bool, &done, "async_release_files"));
+	run_until(ios, done);
 
+	std::cerr << "async_rename_file" << std::endl;
+	done = false;
+	pm->async_rename_file(0, "temp_storage/test1.tmp", boost::bind(&signal_bool, &done, "rename_file"));
+	run_until(ios, done);
+  
 	TEST_CHECK(!exists(test_path / "part0"));	
 	TEST_CHECK(exists(test_path / "temp_storage/test1.tmp"));
 
@@ -568,14 +635,15 @@ namespace
 	{
 		std::cerr << "check_files_fill_array ret: " << ret
 			<< " piece: " << j.piece
+			<< " have: " << j.offset
 			<< " str: " << j.str
+			<< " e: " << j.error.message()
 			<< std::endl;
 
 		if (j.offset >= 0) array[j.offset] = true;
-		if (ret != -1)
+		if (ret != piece_manager::need_full_check)
 		{
 			*done = true;
-			return;
 		}
 	}
 }
@@ -632,21 +700,14 @@ void test_check_files(path const& test_path
 	lazy_entry frd;
 	pm->async_check_fastresume(&frd, boost::bind(&on_check_resume_data, _1, _2, &done));
 	ios.reset();
-	while (!done)
-	{
-		ios.reset();
-		ios.run_one(ec);
-	}
+	run_until(ios, done);
 
 	bool pieces[4] = {false, false, false, false};
-	done = false;
 
-	pm->async_check_files(bind(&check_files_fill_array, _1, _2, pieces, &done));
-	while (!done)
-	{
-		ios.reset();
-		ios.run_one(ec);
-	}
+	done = false;
+	pm->async_check_files(boost::bind(&check_files_fill_array, _1, _2, pieces, &done));
+	run_until(ios, done);
+
 	TEST_CHECK(pieces[0] == true);
 	TEST_CHECK(pieces[1] == false);
 	TEST_CHECK(pieces[2] == false);
@@ -931,10 +992,10 @@ int test_main()
 		}
 	}
 
-	std::for_each(test_paths.begin(), test_paths.end(), bind(&test_fastresume, _1));
-	std::for_each(test_paths.begin(), test_paths.end(), bind(&test_rename_file_in_fastresume, _1));
-	std::for_each(test_paths.begin(), test_paths.end(), bind(&run_test, _1, true));
-	std::for_each(test_paths.begin(), test_paths.end(), bind(&run_test, _1, false));
+	std::for_each(test_paths.begin(), test_paths.end(), boost::bind(&test_fastresume, _1));
+	std::for_each(test_paths.begin(), test_paths.end(), boost::bind(&test_rename_file_in_fastresume, _1));
+	std::for_each(test_paths.begin(), test_paths.end(), boost::bind(&run_test, _1, true));
+	std::for_each(test_paths.begin(), test_paths.end(), boost::bind(&run_test, _1, false));
 
 	return 0;
 }
